@@ -1,10 +1,14 @@
+import re
+import sys
+import threading
+
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 import PyQt4.QtCore as QtCore
 
 import electrum
 from electrum.i18n import _
-from electrum import Wallet, Wallet_2of2, Wallet_2of3
+from electrum import Wallet
 from electrum import bitcoin
 from electrum import util
 
@@ -13,29 +17,63 @@ from network_dialog import NetworkDialog
 from util import *
 from amountedit import AmountEdit
 
-import sys
-import threading
-from electrum.plugins import run_hook
+from electrum.plugins import always_hook, run_hook
 from electrum.mnemonic import prepare_seed
 
-MSG_ENTER_ANYTHING    = _("Please enter a wallet seed, a master public key, a list of Bitcoin addresses, or a list of private keys")
-MSG_SHOW_MPK          = _("This is your master public key")
+MSG_ENTER_ANYTHING    = _("Please enter a seed phrase, a master key, a list of Bitcoin addresses, or a list of private keys")
+MSG_SHOW_MPK          = _("Here is your master public key")
 MSG_ENTER_MPK         = _("Please enter your master public key")
-MSG_ENTER_COLD_MPK    = _("Please enter the master public key of your cosigner wallet")
-MSG_ENTER_SEED_OR_MPK = _("Please enter a wallet seed, or master public key")
+MSG_ENTER_SEED_OR_MPK = _("Please enter a seed phrase or a master key (xpub or xprv)")
 MSG_VERIFY_SEED       = _("Your seed is important!") + "\n" + _("To make sure that you have properly saved your seed, please retype it here.")
 
 
+class CosignWidget(QWidget):
+    size = 120
+
+    def __init__(self, m, n):
+        QWidget.__init__(self)
+        self.R = QRect(0, 0, self.size, self.size)
+        self.setGeometry(self.R)
+        self.m = m
+        self.n = n
+
+    def set_n(self, n):
+        self.n = n
+        self.update()
+
+    def set_m(self, m):
+        self.m = m
+        self.update()
+
+    def paintEvent(self, event):
+        import math
+        bgcolor = self.palette().color(QPalette.Background)
+        pen = QPen(bgcolor, 7, QtCore.Qt.SolidLine)
+        qp = QPainter()
+        qp.begin(self)
+        qp.setPen(pen)
+        qp.setRenderHint(QPainter.Antialiasing)
+        qp.setBrush(Qt.gray)
+        for i in range(self.n):
+            alpha = int(16* 360 * i/self.n)
+            alpha2 = int(16* 360 * 1/self.n)
+            qp.setBrush(Qt.green if i<self.m else Qt.gray)
+            qp.drawPie(self.R, alpha, alpha2)
+        qp.end()
+
+
+
 class InstallWizard(QDialog):
 
-    def __init__(self, config, network, storage):
+    def __init__(self, config, network, storage, app):
         QDialog.__init__(self)
+        self.app = app
         self.config = config
         self.network = network
         self.storage = storage
         self.setMinimumSize(575, 400)
         self.setMaximumSize(575, 400)
-        self.setWindowTitle('Electrum' + '  -  ' + os.path.basename(self.storage.path))
+        self.setWindowTitle('Electrum' + '  -  ' + _('Install Wizard'))
         self.connect(self, QtCore.SIGNAL('accept'), self.accept)
         self.stack = QStackedLayout()
         self.setLayout(self.stack)
@@ -89,12 +127,10 @@ class InstallWizard(QDialog):
                 button.setChecked(True)
 
         vbox.addStretch(1)
-        hbox, button = ok_cancel_buttons2(self, _('Next'))
-        vbox.addLayout(hbox)
         self.set_layout(vbox)
+        vbox.addLayout(Buttons(CancelButton(self), OkButton(self, _('Next'))))
         self.show()
         self.raise_()
-        button.setDefault(True)
 
         if not self.exec_():
             return None, None
@@ -131,8 +167,8 @@ class InstallWizard(QDialog):
             func = self.is_any
         vbox, seed_e = seed_dialog.enter_seed_box(msg, self, sid)
         vbox.addStretch(1)
-        hbox, button = ok_cancel_buttons2(self, _('Next'))
-        vbox.addLayout(hbox)
+        button = OkButton(self, _('Next'))
+        vbox.addLayout(Buttons(CancelButton(self), button))
         button.setEnabled(False)
         seed_e.textChanged.connect(lambda: button.setEnabled(func(self.get_seed_text(seed_e))))
         self.set_layout(vbox)
@@ -143,16 +179,28 @@ class InstallWizard(QDialog):
 
     def multi_mpk_dialog(self, xpub_hot, n):
         vbox = QVBoxLayout()
+        scroll = QScrollArea()
+        scroll.setEnabled(True)
+        scroll.setWidgetResizable(True)
+        vbox.addWidget(scroll)
+
+        w = QWidget()
+        scroll.setWidget(w)
+
+        innerVbox = QVBoxLayout()
+        w.setLayout(innerVbox)
+
         vbox0 = seed_dialog.show_seed_box(MSG_SHOW_MPK, xpub_hot, 'hot')
-        vbox.addLayout(vbox0)
+        innerVbox.addLayout(vbox0)
         entries = []
         for i in range(n):
-            vbox2, seed_e2 = seed_dialog.enter_seed_box(MSG_ENTER_COLD_MPK, self, 'cold')
-            vbox.addLayout(vbox2)
+            msg = _("Please enter the master public key of cosigner") + ' %d'%(i+1)
+            vbox2, seed_e2 = seed_dialog.enter_seed_box(msg, self, 'cold')
+            innerVbox.addLayout(vbox2)
             entries.append(seed_e2)
         vbox.addStretch(1)
-        hbox, button = ok_cancel_buttons2(self, _('Next'))
-        vbox.addLayout(hbox)
+        button = OkButton(self, _('Next'))
+        vbox.addLayout(Buttons(CancelButton(self), button))
         button.setEnabled(False)
         f = lambda: button.setEnabled( map(lambda e: Wallet.is_xpub(self.get_seed_text(e)), entries) == [True]*len(entries))
         for e in entries:
@@ -165,31 +213,37 @@ class InstallWizard(QDialog):
 
     def multi_seed_dialog(self, n):
         vbox = QVBoxLayout()
+        scroll = QScrollArea()
+        scroll.setEnabled(True)
+        scroll.setWidgetResizable(True)
+        vbox.addWidget(scroll)
+
+        w = QWidget()
+        scroll.setWidget(w)
+
+        innerVbox = QVBoxLayout()
+        w.setLayout(innerVbox)
+
         vbox1, seed_e1 = seed_dialog.enter_seed_box(MSG_ENTER_SEED_OR_MPK, self, 'hot')
-        vbox.addLayout(vbox1)
+        innerVbox.addLayout(vbox1)
         entries = [seed_e1]
         for i in range(n):
             vbox2, seed_e2 = seed_dialog.enter_seed_box(MSG_ENTER_SEED_OR_MPK, self, 'cold')
-            vbox.addLayout(vbox2)
+            innerVbox.addLayout(vbox2)
             entries.append(seed_e2)
         vbox.addStretch(1)
-        hbox, button = ok_cancel_buttons2(self, _('Next'))
-        vbox.addLayout(hbox)
+        button = OkButton(self, _('Next'))
+        vbox.addLayout(Buttons(CancelButton(self), button))
         button.setEnabled(False)
-
         f = lambda: button.setEnabled( map(lambda e: self.is_any(self.get_seed_text(e)), entries) == [True]*len(entries))
         for e in entries:
             e.textChanged.connect(f)
-
         self.set_layout(vbox)
         if not self.exec_():
             return
         return map(lambda e: self.get_seed_text(e), entries)
 
 
-
-
-
     def waiting_dialog(self, task, msg= _("Electrum is generating your addresses, please wait.")):
         def target():
             task()
@@ -239,7 +293,7 @@ class InstallWizard(QDialog):
         vbox.addLayout(grid)
 
         vbox.addStretch(1)
-        vbox.addLayout(ok_cancel_buttons(self, _('Next')))
+        vbox.addLayout(Buttons(CancelButton(self), OkButton(self, _('Next'))))
 
         self.set_layout(vbox)
         if not self.exec_():
@@ -248,7 +302,7 @@ class InstallWizard(QDialog):
         if b2.isChecked():
             return NetworkDialog(self.network, self.config, None).do_exec()
         else:
-            self.config.set_key('auto_cycle', True, True)
+            self.config.set_key('auto_connect', True, True)
             return
 
 
@@ -261,7 +315,7 @@ class InstallWizard(QDialog):
             vbox.addWidget(logo)
         vbox.addWidget(QLabel(msg))
         vbox.addStretch(1)
-        vbox.addLayout(close_button(self, _('Next')))
+        vbox.addLayout(Buttons(CloseButton(self)))
         if not self.exec_():
             return None
 
@@ -282,13 +336,55 @@ class InstallWizard(QDialog):
             if i==0:
                 button.setChecked(True)
         vbox.addStretch(1)
-        vbox.addLayout(ok_cancel_buttons(self, _("Next")))
+        vbox.addLayout(Buttons(CancelButton(self), OkButton(self, _('Next'))))
         if not self.exec_():
             return
         wallet_type = choices[group2.checkedId()][0]
         return wallet_type
 
 
+    def multisig_choice(self):
+
+        vbox = QVBoxLayout()
+        self.set_layout(vbox)
+        vbox.addWidget(QLabel(_("Multi Signature Wallet")))
+
+        cw = CosignWidget(2, 2)
+        vbox.addWidget(cw, 1)
+        vbox.addWidget(QLabel(_("Please choose the number of signatures needed to unlock funds in your wallet") + ':'))
+
+        m_edit = QSpinBox()
+        n_edit = QSpinBox()
+        m_edit.setValue(2)
+        n_edit.setValue(2)
+        n_edit.setMinimum(2)
+        n_edit.setMaximum(15)
+        m_edit.setMinimum(1)
+        m_edit.setMaximum(2)
+        n_edit.valueChanged.connect(m_edit.setMaximum)
+
+        n_edit.valueChanged.connect(cw.set_n)
+        m_edit.valueChanged.connect(cw.set_m)
+
+        hbox = QHBoxLayout()
+        hbox.addWidget(QLabel(_('Require')))
+        hbox.addWidget(m_edit)
+        hbox.addWidget(QLabel(_('of')))
+        hbox.addWidget(n_edit)
+        hbox.addWidget(QLabel(_('signatures')))
+        hbox.addStretch(1)
+
+        vbox.addLayout(hbox)
+        vbox.addStretch(1)
+        vbox.addLayout(Buttons(CancelButton(self), OkButton(self, _('Next'))))
+        if not self.exec_():
+            return
+        m = int(m_edit.value())
+        n = int(n_edit.value())
+        wallet_type = '%dof%d'%(m,n)
+        return wallet_type
+
+
     def question(self, msg, yes_label=_('OK'), no_label=_('Cancel'), icon=None):
         vbox = QVBoxLayout()
         self.set_layout(vbox)
@@ -300,7 +396,7 @@ class InstallWizard(QDialog):
         label.setWordWrap(True)
         vbox.addWidget(label)
         vbox.addStretch(1)
-        vbox.addLayout(ok_cancel_buttons(self, yes_label, no_label))
+        vbox.addLayout(Buttons(CancelButton(self, no_label), OkButton(self, yes_label)))
         if not self.exec_():
             return None
         return True
@@ -308,7 +404,7 @@ class InstallWizard(QDialog):
 
     def show_seed(self, seed, sid):
         vbox = seed_dialog.show_seed_box_msg(seed, sid)
-        vbox.addLayout(ok_cancel_buttons(self, _("Next")))
+        vbox.addLayout(Buttons(CancelButton(self), OkButton(self, _("Next"))))
         self.set_layout(vbox)
         return self.exec_()
 
@@ -321,15 +417,11 @@ class InstallWizard(QDialog):
         return run_password_dialog(self, None, self)[2]
 
 
+    def run(self, action, wallet_type):
 
-
-
-    def run(self, action):
-
-        if action == 'new':
-            action, wallet_type = self.restore_or_create()
+        if action in ['create', 'restore']:
             if wallet_type == 'multisig':
-                wallet_type = self.choice(_("Multi Signature Wallet"), 'Select wallet type', [('2of2', _("2 of 2")),('2of3',_("2 of 3"))])
+                wallet_type = self.multisig_choice()
                 if not wallet_type:
                     return
             elif wallet_type == 'hardware':
@@ -339,7 +431,6 @@ class InstallWizard(QDialog):
                     return
             elif wallet_type == 'twofactor':
                 wallet_type = '2fa'
-
             if action == 'create':
                 self.storage.put('wallet_type', wallet_type, False)
 
@@ -357,11 +448,15 @@ class InstallWizard(QDialog):
             # fixme: password is only needed for multiple accounts
             password = None
 
+        # load wallet in plugins
+        always_hook('installwizard_load_wallet', wallet, self)
+
         while action is not None:
             util.print_error("installwizard:", wallet, action)
 
             if action == 'create_seed':
-                seed = wallet.make_seed()
+                lang = self.config.get('language')
+                seed = wallet.make_seed(lang)
                 if not self.show_seed(seed, None):
                     return
                 if not self.verify_seed(seed, None):
@@ -370,29 +465,21 @@ class InstallWizard(QDialog):
                 wallet.add_seed(seed, password)
                 wallet.create_master_keys(password)
 
-            elif action == 'add_cosigner':
+            elif action == 'add_cosigners':
+                n = int(re.match('(\d+)of(\d+)', wallet.wallet_type).group(2))
                 xpub1 = wallet.master_public_keys.get("x1/")
-                r = self.multi_mpk_dialog(xpub1, 1)
+                r = self.multi_mpk_dialog(xpub1, n - 1)
                 if not r:
                     return
-                xpub2 = r[0]
-                wallet.add_master_public_key("x2/", xpub2)
-
-            elif action == 'add_two_cosigners':
-                xpub1 = wallet.master_public_keys.get("x1/")
-                r = self.multi_mpk_dialog(xpub1, 2)
-                if not r:
-                    return
-                xpub2, xpub3 = r
-                wallet.add_master_public_key("x2/", xpub2)
-                wallet.add_master_public_key("x3/", xpub3)
+                for i, xpub in enumerate(r):
+                    wallet.add_master_public_key("x%d/"%(i+2), xpub)
 
             elif action == 'create_accounts':
                 wallet.create_main_account(password)
                 self.waiting_dialog(wallet.synchronize)
 
             else:
-                f = run_hook('get_wizard_action', self, wallet, action)
+                f = always_hook('get_wizard_action', self, wallet, action)
                 if not f:
                     raise BaseException('unknown wizard action', action)
                 r = f(wallet, self)
@@ -446,86 +533,25 @@ class InstallWizard(QDialog):
                     wallet = Wallet.from_private_key(text, password, self.storage)
                 elif Wallet.is_seed(text):
                     password = self.password_dialog()
-                    wallet = Wallet.from_seed(text, self.storage)
-                    wallet.add_seed(text, password)
-                    wallet.create_master_keys(password)
-                    wallet.create_main_account(password)
+                    wallet = Wallet.from_seed(text, password, self.storage)
                 else:
                     raise BaseException('unknown wallet type')
 
-            elif t in ['2of2']:
-                r = self.multi_seed_dialog(1)
-                if not r:
+            elif re.match('(\d+)of(\d+)', t):
+                n = int(re.match('(\d+)of(\d+)', t).group(2))
+                key_list = self.multi_seed_dialog(n - 1)
+                if not key_list:
                     return
-                text1, text2 = r
-                wallet = Wallet_2of2(self.storage)
-                if Wallet.is_seed(text1) or Wallet.is_seed(text2):
-                    password = self.password_dialog()
-                else:
-                    password = None
-
-                if Wallet.is_seed(text2) and Wallet.is_xpub(text1):
-                    c = text1
-                    text1 = text2
-                    text2 = c
-
-                if Wallet.is_seed(text1):
-                    wallet.add_seed(text1, password)
-                    wallet.create_master_keys(password)
-                else:
-                    wallet.add_master_public_key("x1/", text1)
-
-                if Wallet.is_seed(text2):
-                    wallet.add_cosigner_seed(text2, "x2/", password)
-                elif Wallet.is_xpub(text2):
-                    wallet.add_master_public_key("x2/", text2)
-
-                wallet.create_main_account(password)
-
-
-            elif t in ['2of3']:
-                r = self.multi_seed_dialog(2)
-                if not r:
-                    return
-                text1, text2, text3 = r
-                wallet = Wallet_2of3(self.storage)
-                if Wallet.is_seed(text1) or Wallet.is_seed(text2) or Wallet.is_seed(text3):
-                    password = self.password_dialog()
-                else:
-                    password = None
-
-                if Wallet.is_xpub(text1) and Wallet.is_seed(text2):
-                    temp = text1
-                    text1 = text2
-                    text2 = temp
-
-                if Wallet.is_xpub(text1) and Wallet.is_seed(text3):
-                    temp = text1
-                    text1 = text3
-                    text3 = temp
-
-                if Wallet.is_seed(text1):
-                    wallet.add_seed(text1, password)
-                    wallet.create_master_keys(password)
-                else:
-                    wallet.add_master_public_key("x1/", text1)
-
-                if Wallet.is_seed(text2):
-                    wallet.add_cosigner_seed(text2, "x2/", password)
-                elif Wallet.is_xpub(text2):
-                    wallet.add_master_public_key("x2/", text2)
-
-                if Wallet.is_seed(text3):
-                    wallet.add_cosigner_seed(text3, "x3/", password)
-                elif Wallet.is_xpub(text3):
-                    wallet.add_master_public_key("x3/", text3)
-
-                wallet.create_main_account(password)
+                password = self.password_dialog() if any(map(lambda x: Wallet.is_seed(x) or Wallet.is_xprv(x), key_list)) else None
+                wallet = Wallet.from_multisig(key_list, password, self.storage, t)
 
             else:
-                self.storage.put('wallet_type', t)
-                wallet = run_hook('installwizard_restore', self, self.storage)
+                self.storage.put('wallet_type', t, False)
+                # call the constructor to load the plugin (side effect)
+                Wallet(self.storage)
+                wallet = always_hook('installwizard_restore', self, self.storage)
                 if not wallet:
+                    util.print_error("no wallet")
                     return
 
             # create first keys offline
